##---------------------------------------------------------------------
## Debug_Window ゲーム画面上に状態表示、ログ表示などを行う
##---------------------------------------------------------------------

class Debug_Window
  FONT_SIZE = 16
  WINDOW_Z = 10000
  WINDOW_BACKGROUND_ALPHA = 0x80
  ACTIVATE_KEY = Input::F5
  MODE_INACTIVE = 0
  MODE_STATE    = 1
  MODE_LOG      = 2
  ##---------------------------------------------------------------------
  ## 初期化
  ##   enable   trueでDebug_Window表示。falseで全機能停止(Release用)
  ##   dx       表示X座標
  ##   dy       表示Y座標
  ##   width    幅（dot）
  ##   line_max 高さ（行数）
  ##   mode     開始時のモード
  ##---------------------------------------------------------------------
  def initialize(enable, dx = 0, dy = 0, width = 640, line_max = 8, mode = MODE_INACTIVE)
    @enable = enable
    if @enable
      @dx = dx
      @dy = dy
      @width = width
      @line_max = line_max
      @mode = mode

      @line_height = FONT_SIZE + 2
      @height = 2 + @line_height * @line_max + 2

      @state = [] # state文字列を格納
      @log = [] # log文字列を格納
      @log_max = 0
      @log_disp_y = 0
      @log_changed = false

      r = Rect.new(@dx, @dy, @width, @height)
      ##-------------- 背景 --------------
      @win_back = Sprite.new
      @win_back.bitmap = Bitmap.new(@width, @height)
      @win_back.bitmap.fill_rect(r, Color.new(0, 0, 0, WINDOW_BACKGROUND_ALPHA))
      @win_back.src_rect = r
      @win_back.x = @dx
      @win_back.y = @dy
      @win_back.z = WINDOW_Z
      ##-------------- フォント --------------
      @win_font = Sprite.new
      @win_font.bitmap = Bitmap.new(@width, @height)
      @win_font_back_color = Color.new(0, 0, 0, 0)
      @win_font.bitmap.fill_rect(r, Color.new(0, 0, 0, 0))
      @win_font.bitmap.font.name = "ＭＳ ゴシック"
      @win_font.bitmap.font.size = FONT_SIZE
      @win_font.bitmap.font.color = Color.new(0xff, 0xff, 0xff, 0xff)
      @win_font.src_rect = r
      @win_font.x = @dx
      @win_font.y = @dy
      @win_font.z = WINDOW_Z + 1
      # mode
      set_mode
    end
  end
  ##---------------------------------------------------------------------
  ## 廃棄 
  ##---------------------------------------------------------------------
  def dispose
    if @enable
      @win_font.bitmap.dispose
      @win_font.dispose
      @win_back.bitmap.dispose
      @win_back.dispose
    end
  end
  ##---------------------------------------------------------------------
  ## モード変更 
  ##---------------------------------------------------------------------
  def set_mode
    @win_back.visible = @win_font.visible = true
    if @mode == MODE_INACTIVE
      @win_back.visible = @win_font.visible = false
    end
    if @mode == MODE_STATE
      refresh_state
    elsif @mode == MODE_LOG
      refresh_log
    end
  end
  ##---------------------------------------------------------------------
  ## 更新 
  ##---------------------------------------------------------------------
  def update
    if @enable
      if Input.trigger?(ACTIVATE_KEY) && !Input.press?(Input::SHIFT) && !Input.press?(Input::CTRL)
        ##-------------- モード変更キーが押された --------------
        case @mode
        when MODE_INACTIVE
          @mode = MODE_STATE
        when MODE_STATE
          @mode = MODE_LOG
        when MODE_LOG
          @mode = MODE_INACTIVE
        end
        set_mode
      else
        ##-------------- 通常時の更新処理 --------------
        case @mode
        when MODE_STATE
          refresh_state # stateは毎フレーム更新
        when MODE_LOG
          if Input.press?(Input::SHIFT)
            if Input.repeat?(ACTIVATE_KEY)
              scroll_log(-1)
            end
          end
          if Input.press?(Input::CTRL)
            if Input.repeat?(ACTIVATE_KEY)
              scroll_log(1)
            end
          end
          if @log_changed
            refresh_log # logは内容に変更があれば更新
          end
        end
      end
    end
  end
  ##---------------------------------------------------------------------
  ## ステートに追加 
  ##---------------------------------------------------------------------
  def state(n, s, *p)
    if @enable
      @state[n] = sprintf(s, *p)
    end
  end
  ##---------------------------------------------------------------------
  ## ステートに追加 
  ##---------------------------------------------------------------------
  def s(n, s, *p)
    state(n, s, *p)
  end
  ##---------------------------------------------------------------------
  ## ステート画面書き換え 
  ##---------------------------------------------------------------------
  def refresh_state
    @win_font.bitmap.clear
    x = 2
    y = 2
    @state.each do |s|
      if s != nil
        @win_font.bitmap.draw_text(x, y, @width - x, FONT_SIZE, s)
      else
        @win_font.bitmap.fill_rect(x, y, @width - x, FONT_SIZE, @win_font_back_color)
      end
      y += @line_height
    end
    @state.clear
  end
  ##---------------------------------------------------------------------
  ## ログに追加 
  ##---------------------------------------------------------------------
  def log(s, *p)
    if @enable
      @log[@log_max] = sprintf(s, *p)
      @log_max += 1
      if (@log_max > @line_max) && (@log_max - @line_max == @log_disp_y + 1)
        # ログの一番下を表示している（スクロールしていない）なら
        @log_disp_y += 1 # １行スクロール
      end
      @log_changed = true
    end
  end
  ##---------------------------------------------------------------------
  ## ログに追加 
  ##---------------------------------------------------------------------
  def l(s, *p)
    log(s, *p)
  end
  ##---------------------------------------------------------------------
  ## ログ画面書き換え 
  ##---------------------------------------------------------------------
  def refresh_log
    @win_font.bitmap.clear
    x = 2
    y = 2
    i = @log_disp_y
    j = 0
    while ((i < @log_max) && (j < @line_max))
      @win_font.bitmap.draw_text(x, y, @width - x, FONT_SIZE, @log[i])
      i += 1
      j += 1
      y += @line_height
    end
    @log_changed = false
  end
  ##---------------------------------------------------------------------
  ## ログ画面スクロール 
  ##---------------------------------------------------------------------
  def scroll_log(n)
    temp = @log_disp_y
    @log_disp_y += n
    if @log_disp_y < 0
      @log_disp_y = 0
    end
    if (@log_disp_y + @line_max) > @log_max
      @log_disp_y = @log_max - @line_max
      if @log_disp_y < 0
        @log_disp_y = 0
      end
    end
    if temp != @log_disp_y
      @log_changed = true
    end
  end
end

